import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class AreaDesenho extends JPanel implements MouseMotionListener
{
    Aplicacao aplic; // ref. para aplicao principal
    ListaDeFiguras lista;
    int sel;   // bola selecionada ou -1 se nenhuma
    int clickx,clicky; // posio inicial do click

    public AreaDesenho(Aplicacao aplic,int w,int h)
    {
        super();
        this.aplic = aplic;
        sel = -1;
        lista = new ListaDeFiguras(100);
        setPreferredSize(new Dimension(w,h));
        addMouseListener(
          new MouseAdapter()
          {
            public void mousePressed(MouseEvent e)
            {
                novoPonto(e.getX(),e.getY());
            }
          });
        // Para tratar mouse drag (arrasto)
        addMouseMotionListener(this);
    }

    public void paintComponent(Graphics g)
    {
        int ox1,ox2,oy1,oy2;
        super.paintComponent(g); // pinta o fundo       
        g.setColor(Color.red);
        for(int i=0;i<lista.total();++i)
        {
           g.setColor(Color.black);
           // Desenha linha ao ponto anterior
           Figura fig2 = lista.retorna(i);
           ox2 = fig2.getX();
           oy2 = fig2.getY();
           if(i>0)
           {
                Figura fig1 = lista.retorna(i-1);
                ox1 = fig1.getX();
                oy1 = fig1.getY();
                g.drawLine(ox1,oy1,ox2,oy2);
           }
           // Se for o selecionado, desenha bola vermelha
           if(i==sel) g.setColor(Color.red);
           // ... seno, azul
            else g.setColor(Color.blue);
           fig2.desenha(g);       
        }
    }

    public void novoPonto(int x, int y)
    {
        int c;
        Figura obj;
        int ox,oy,tam;
        // Verifica se clicou sobre um ponto existente
        for(c=0;c<lista.total();++c)
        {
            obj = lista.retorna(c);
            ox = obj.getX();
            oy = obj.getY();
            tam = obj.getTam();  // pega tam. do objeto
            if(ox >= x-tam && ox <= x+tam &&
               oy >= y-tam && oy <= y+tam)
            {
                // Sim, seta como selecionado,
                // redesenha a tela e retorna
                sel = c;
                // Salva posio inicial do click
                // (para redimensionar, se for o caso)
                clickx = ox;
                clicky = oy;
                if(aplic.getObj()==3) // apagar ?
                {
                    lista.remove(c);
                    sel = -1;
                }
                repaint();
                return;
            }
        }
        // Adiciona novo objeto e redesenha
        switch(aplic.getObj())
        {
            case 0:  lista.insere(new Quadrado(x,y));
                     break;
            case 1:  lista.insere(new Circulo(x,y));
                     break;
            case 2:  lista.insere(new Triang(x,y));
                     break;
        }
        sel = -1;
        aplic.atualizaCont(lista.total());
        repaint();
    }

    // Chamada quando o mouse for arrastado e o boto
    // estiver pressionado
    public void mouseDragged(MouseEvent e)
    {
        int x2,y2;
        Dimension tam = new Dimension();
        // Se no houver ningum selecionado, retorna
        if(sel==-1) return;
        // Obtm as coords atuais do mouse
        x2 = e.getX(); y2 = e.getY();
        // Compara com as dimenses da janela:
        // Se for superior em x/y ou menor que 0,
        // ajusta para o mximo/mnimo
        getSize(tam);
        if(x2<0) x2=0;
        if(x2>tam.width) x2 = (int) tam.width;
        if(y2<0) y2=0;
        if(y2>tam.height) y2 = (int) tam.height;
        Figura fig = lista.retorna(sel);
        // Se estiver no modo de redimensionar,
        // altera dimenses do objeto selecionado
        if(aplic.getObj()==4)
        {
            fig.setTam(Math.abs(clickx-x2));
        }
        // seno, atualiza a posio do ponto selecionado
        else fig.setPos(x2,y2);
        // Redesenha
        repaint();
    }

    // Chamada quando houver movimento do mouse,
    // no  necessria aqui
    public void mouseMoved(MouseEvent e) {}

}